/* muck2html (c) Lawrie.Brown@adfa.oz.au 1994                              */
/*   - convert a muck database file into html for browsing using the WWW   */
/*                                                                         */
/*   v1.0 by Lawrie.Brown@adfa.oz.au 25 Aug 1994                           */
/*        - 1.01 lpb 26 aug, fiddle with exit handling                     */
/*                                                                         */
/*   Mangled from sanity.c,v 2.16 by David Moore 1994                      */
/*     and inspired by wld2html by Jeremy Elson 1994                       */

#include "config.h"

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "ansify.h"
#include "db.h"
#include "params.h"
#include "buffer.h"
#include "externs.h"

static char usage[] = "[-i] [-n mudname] [-o dbref]";

/* special macros for database processing - stolen from sanity.c */
#undef Typeof
#define Typeof(x) \
    ((x) == HOME ? TYPE_ROOM : ((x) < 0 || (x) >= db_top)  ? TYPE_GARBAGE : (GetFlags(x) & TYPE_MASK))

#undef DOLIST
#define DOLIST(v, first) \
    for ((v) = (first); (v) != NOTHING && ok_dbref(v); (v) = GetNext(v))

/* Global flags */
int oneshot = 0;
int use_icon = 0;

/* Global data */
char *mudname = "Muck";

/* define for the muckup program name, used for hotlinks in oneshot mode */
#define MUCKUP "muckup"

/* defines for default icon names for various things */
#define I_EXIT		"exit.gif"
#define I_MALE		"male.gif"
#define I_FEMALE	"female.gif"
#define I_PLAYER	"player.gif"
#define I_ROOM		"room.gif"
#define I_THING		"thing.gif"
#define I_WIZ		"wiz.gif"

/* This routine verifies that the object it is passed is ''ok''.
   That means, it must be within the legal range of dbrefs, and be of
   normal object type. */
static int ok_dbref(const dbref obj)
{
    if (obj >= db_top) return 0;
    
    /* obj < db_top */
    if (obj >= 0) {
        return ((Typeof(obj) == TYPE_ROOM)
            || (Typeof(obj) == TYPE_PLAYER)
            || (Typeof(obj) == TYPE_THING)
            || (Typeof(obj) == TYPE_EXIT)
            || (Typeof(obj) == TYPE_PROGRAM));
    }
    
    /* obj < 0 */
    return (0);
}

/* first_name - copy name into nambuf up to first ; or end of string */
/*              used to extract first name component from exit name  */
char *first_name(const char *name)
{
    static char namebuf[128];    /* buffer returned with first name component */
    int j;

    for (j=0; ((name[j] != '\0') && (name[j] != ';')) ; j++) {
        namebuf[j] = name[j];
    }
    namebuf[j] = '\0';
    return(namebuf);
}


/* get_icon - determine icon for dbref if wanted & return inline ref */
char *get_icon(const dbref obj)
{
    static char iconbuf[128];    /* buffer returned with inline html icon */
    char	*iname;

    if (!use_icon)	return "";	/* not using icons, so forget it */

    /* see if Icon property defined, if so use that name, otherwise default */
    if ((iname = get_string_prop(obj, "Icon", NORMAL_PROP)) == NULL) {

	/* apply some heuristics to guess an appropriate default icon */
        switch (Typeof(obj)) {
            case TYPE_PLAYER:
		if (Wizard(obj))
		    iname = I_WIZ;
		else 
		    iname = I_PLAYER;
		break;
            case TYPE_THING:
		iname = I_THING;
		break;
            case TYPE_EXIT:
		iname = I_EXIT;
		break;
            case TYPE_ROOM:
		iname = I_ROOM;
		break;
        }
    }

    if (!oneshot) {	/* if not oneshot, just name icon file */
        sprintf(iconbuf, "<img src=\"%s\">", iname);
    } else {		/* if oneshot, icons relative to document root/mudnam */
        sprintf(iconbuf, "<img src=\"/%s/%s\">", mudname, iname);
    }

    return iconbuf;
}

/* do_room - create html output describing this room */
static void do_room(const dbref obj)
{
    dbref i;
    int   ndest;
    dbref *exits;
    FILE *fl;
    char *icon, *str;
    char buf[128];

    if (!oneshot) {
        /* open output file for this room */
        sprintf(buf, "%d.html", obj);
        if (!(fl = fopen(buf, "w"))) {
            perror("opening output file");
            exit(1);
        }
    } else {
    /* point to stdout for oneshot dbref */
        fl = stdout;
    }

    /* write html describing this room to the output file */
    fprintf(fl, "<html>\n");
    fprintf(fl, "<title>%s</title>\n", GetName(obj));
    icon = get_icon(obj);
    fprintf(fl, "%s<h1>%s</h1>\n", icon, GetName(obj));
    fprintf(fl, "<p>\n");

    /* get description of object, if any, and display it */
    if ((str = GetDesc(obj)) != NULL) {
        fprintf(fl, "%s\n", str);
    }

    /* process contents of this room provided its not dark */
    if (!Dark(obj) && (GetContents(obj) != NOTHING)) {
        fprintf(fl, "<h2>Contents:</h2>\n<dl>\n");
        DOLIST(i, GetContents(obj)) {
            if (ok_dbref(i) && !Dark(i)) {    /* item ok & not dark */
                switch (Typeof(i)) {
                    case TYPE_PLAYER:
                        icon = get_icon(i);
                        fprintf(fl, "<dt> %s%s\n", icon, GetName(i));
                        if ((str = GetDesc(i)) != NULL) 
                            fprintf(fl, "<dd> %s\n", str);
                        break;
                    case TYPE_THING:
                        icon = get_icon(i);
                        fprintf(fl, "<dt> %s%s\n", icon, GetName(i));
                        if ((str = GetDesc(i)) != NULL) 
                            fprintf(fl, "<dd> %s\n", str);
                        break;
                }
            }
        }
        fprintf(fl, "</dl>\n");
    }

    /* process exits for this room */
    fprintf(fl, "<h2>Exits: </h2>\n<dir>\n");
    DOLIST(i, GetActions(obj)) {
        ndest = GetNDest(i);    /* get exit list */
        exits = GetDest(i);
        icon = get_icon(i);	/* get icon for this exit */
        /* if exit ok & not dark exit & not dark dest & dest is a room */
        if (ok_dbref(*exits) && !Dark(i) && !Dark(*exits) &&
	         (Typeof(*exits) == TYPE_ROOM)) {
            /* display this exit and its first destination as hotlink */
	    if (!oneshot) {	/* normally a ref to another room file */
                fprintf(fl, "<li> <a href = \"%ld.html\"> %s%s</a>",
                    *exits, icon, first_name(GetName(i)));
	    } else {		/* oneshot mode uses ref to muckup script */
                fprintf(fl, "<li> <a href = \"%s?%ld\"> %s%s</a>",
                    MUCKUP, *exits, icon, first_name(GetName(i)));
            }
            if ((str = GetDesc(i)) != NULL) 	/* show exit description */
                fprintf(fl, " - %s", str);
            fprintf(fl, "</li>\n");		/* finish exit html item */
        }
    }
    fprintf(fl, "</dir>\n");

    /* tidy up html */
    fprintf(fl, "<p><hr><address>%s room #%ld</address>\n", mudname, obj);
    fprintf(fl, "</html>\n");

    if (!oneshot) {
        /* close output file for this room */
        fclose(fl);
    }
}


int main(const int argc, const char * const *argv)
{
    dbref i = -1;
    int j;
    FILE	*dbf;		/* file to read MUCK DB from if name given */
    int		have_dbname = 0;	/* flag that db name was given */

    /* scan for input args */
    for (j=1; j<argc; j++) {
        if (argv[j][0] == '-') {
            switch(argv[j][1]) {
                case 'i':
                    use_icon = 1;
                    break;
                case 'n':
                    mudname = argv[++j];
		    have_dbname = 1;
                    break;
                case 'o':
                    i = atoi(argv[++j]);
                    oneshot = 1;
                    break;
                default:
                    fprintf(stderr, "Usage:  %s %s\n", argv[0], usage);
                    exit(1);
            }
        }
    }

    /* open MUCK database if name supplied */
    if (have_dbname) {
        if (!(dbf = fopen(mudname, "r"))) {
            perror("opening MUCK database file");
            exit(1);
        }
        /* read in the muck database */
        db_read(dbf);
	fclose(dbf);
    } else {
        /* read in the muck database from stdin */
        db_read(stdin);
    }

    if (!oneshot) {
        /* run through database processing dbrefs */
        for (i = 0; i < db_top; i++) {
            if (ok_dbref(i) && (Typeof(i) == TYPE_ROOM)) {
                do_room(i);
            }
        }
    } else { /* just process supplied dbref */
        if (ok_dbref(i) && (Typeof(i) == TYPE_ROOM)) {
            do_room(i);
        } else {    /* oops dud dbref */
            printf("<html><title>Invalid</title>\n");
            printf("<h1>Invalid</h1>\n");
            printf("<p>I'm sorry, but %s #%ld is an invalid dbref</html>\n",
                mudname, i);
        }
    }
    
    exit(0);
}
